<?php

namespace NoccyLabs\App\Trait\Lockfile;


class Lockfile
{

    private string $filename;

    private $lockfd;

    private bool $locked = false;

    private bool $isLockOwner = false;

    public function __construct(string $filename)
    {
        $this->filename = $filename;
    }

    public function __destruct()
    {
        if ($this->locked) {
            $this->release();
        }
    }

    public function acquire(): void
    {
        $this->isLockOwner = !file_exists($this->filename);
        $this->lockfd = fopen($this->filename, "a+");
        $this->locked = flock($this->lockfd, LOCK_EX|LOCK_NB);
        if (!$this->locked) {
            throw new \RuntimeException("Could not claim lockfile");
        }
    }

    public function release(): void
    {
        if ($this->lockfd) {
            flock($this->lockfd, LOCK_UN);
            fclose($this->lockfd);
            if ($this->isLockOwner) {
                unlink($this->filename);
            }
        }
    }
}