<?php

namespace NoccyLabs\App\Trait;

use NoccyLabs\App\Attribute\AppInfo;
use NoccyLabs\App\Trait\Lockfile\Lockfile;
use NoccyLabs\App\Trait\Process\Message;
use NoccyLabs\App\Trait\Process\Process;
use Phar;
use ReflectionClass;

trait ProcessTrait
{

    private array $processList = [];

    private array $messages = [];

    protected function startProcess(array $commandLine, string $processName): void
    {
        $this->processList[$processName] = new Process($processName, $commandLine);
    }

    protected function stopProcess(string $processName): void
    {
        $process = $this->processList[$processName]??null;
        if ($process) {
            $process->terminate();
            unset($this->processList[$processName]);
        }
    }

    protected function updateProcesses(): void
    {
        foreach ($this->processList as $process) {
            $process->update();
            while (($msg = $process->readMessage()) !== null) {
                $this->messages[] = new Message($process->processName, $msg);
            }
        }
    }

    protected function getProcessStatus(): array
    {
        return array_map(
            fn($p) => $p->getStatus(),
            $this->processList
        );
    }

    protected function sendMessage(string $processName, mixed $message): void
    {
        $process = $this->processList[$processName]??null;
        if ($process) {
            $process->sendMessage($message);
        }

    }

    protected function receiveMessage(): ?Message
    {
        return array_shift($this->messages);
    }
}