<?php

namespace NoccyLabs\App\Trait;

use NoccyLabs\App\Attribute\AppInfo;
use ReflectionClass;

trait UsageTrait
{

    protected function usage(): void
    {
        $reflection = new ReflectionClass($this);
        $attributes = $reflection->getAttributes(AppInfo::class);
        
        if ($appinfo = array_shift($attributes)) {
            $appinfo = $appinfo->newInstance();
            $out = sprintf("\e[1m%s\e[0m %s %s\n", $appinfo->name, $appinfo->version, $appinfo->summary?"- {$appinfo->summary}":"");
            if ($appinfo->copyright) {
                $out.= "\e[2m".$appinfo->copyright . ($appinfo->license?" - ":""); 
            }
            if ($appinfo->license) {
                $out.= "Licensed under {$appinfo->license}\e[0m\n";
            } elseif ($appinfo->copyright) {
                $out.= "\e[0m\n";
            }
            $out.= "\n";
            if ($appinfo->description) {
                $descr = is_array($appinfo->description) ? join("\n\n", $appinfo->description) : $appinfo->description;
                $out .= " ".join("\n ", explode("\n", wordwrap($descr, 75)))."\n\n";
            }
        } else {
            $exename = basename($GLOBALS['argv'][0]);
            $out = $exename."\n\n";
        }


        $groups = [];
        $nogroup = [];
        foreach ($this->optionGroups as $name=>$group) {
            $groups[$name] = (object)[
                'group' => $group,
                'items' => []
            ];
        }

        foreach ($this->optionProps as $prop) {
            if ($prop->group) {
                $groups[$prop->group]->items[] = $prop;
            } else {
                $nogroup[] = $prop;
            }
        }

        $out .= sprintf("\e[1mCommon options:\e[0m\n");
        foreach ($nogroup as $prop) {
            $legend = 
                ($prop->option->short ? "-{$prop->option->short}" : "") .
                (($prop->option->short && $prop->option->long) ? "," : "") .
                ($prop->option->long ? "--{$prop->option->long}" : "") .
                ($prop->option->hasValue ? " {" . (is_string($prop->option->hasValue)?$prop->option->hasValue:$prop->property->name) . "}" : "");

            $out .= sprintf("  %-25s %s %s\n", 
                $legend, 
                $prop->option->description,
                $prop->option->envvar?" [\e[36m\${$prop->option->envvar}\e[0m]":""
            );
        }

        foreach ($groups as $group) {
            $out .= sprintf("\n\e[1m%s:\e[0m\n", $group->group);
            foreach ($group->items as $prop) {
                $legend = 
                    ($prop->option->short ? "-{$prop->option->short}" : "") .
                    (($prop->option->short && $prop->option->long) ? "," : "") .
                    ($prop->option->long ? "--{$prop->option->long}" : "") .
                    ($prop->option->hasValue ? " {" . (is_string($prop->option->hasValue)?$prop->option->hasValue:$prop->property->name) . "}" : "");

                $out .= sprintf("  %-25s %s %s\n", 
                    $legend, 
                    $prop->option->description,
                    $prop->option->envvar?" [\e[36m\${$prop->option->envvar}\e[0m]":""
                );
            }
        }

        $out .= "\n";

        if ($appinfo && $appinfo->comments) {
            $comments = is_array($appinfo->comments) ? join("\n\n", $appinfo->comments) : $appinfo->comments;
            $out .= " ".join("\n ", explode("\n", wordwrap($comments, 75)))."\n\n";
        }

        if (!posix_isatty(STDOUT)) {
            $out = preg_replace("<\e\[[0-9;]+?m>", "", $out);
        } else {
            $out = preg_replace("<\{(\w+)\}>", "{\e[4;3m$1\e[24;23m}", $out);
            $out = preg_replace("<(-{1,2}[a-zA-Z0-9-]+)([\s,\)])>", "\e[0;33m$1\e[0m$2", $out);
        }

        echo $out;

    }

}
